# WEB+DB PRESS Vol.83 zsh特集 カスタマイズ例

export LANG=ja_JP.UTF-8

#############################################
# 2章 おすすめ環境設定
#############################################

## zshの基本的な設定
# ディレクトリ名だけでcdする
setopt AUTO_CD

# cd したら自動的にpushdする
setopt AUTO_PUSHD

# 重複したディレクトリを追加しない
setopt PUSHD_IGNORE_DUPS

# emacs風キーバインドにする
bindkey -e
# vi風キーバインドにする
#bindkey -v

# 単語の区切り設定
autoload -Uz select-word-style
select-word-style default
# ここで指定した文字は単語区切りとみなされる
# / も区切りと扱うので、^W でディレクトリ１つ分を削除できる
zstyle ':zle:*' word-chars " /=;@:{},|"
zstyle ':zle:*' word-style unspecified

# プロンプトに関する設定
# パターン1
PROMPT="%F{green}[%n@%m]%f(%*%) %~ %# "

# パターン2
#PROMPT="%F{green}[%n@%m]%f(%*%) %~
#%# "

# パターン3
#PROMPT="%F{green}[%n@%m]%f(%*%) %# "
#RPROMPT="[%~]"

# ^Dでzshを終了しない
setopt IGNORE_EOF

# ^Q/^Sのフローコントロールを無効にする
setopt NO_FLOW_CONTROL

# beep音を鳴らさない
setopt NO_BEEP

# 日本語ファイル名を表示可能にする
setopt PRINT_EIGHT_BIT

# '#' 以降をコメントとして扱う
setopt INTERACTIVE_COMMENTS

# 高機能なワイルドカード展開を使用する
setopt EXTENDED_GLOB


## 補完機能に関する設定
# 補完機能を有効にする
autoload -Uz compinit
compinit

# メニュー選択モード
zstyle ':completion:*:default' menu select=2

# 大文字と小文字を区別しない
zstyle ':completion:*' matcher-list 'm:{a-z}={A-Z}'

# ../ の後は今いるディレクトリを補完しない
zstyle ':completion:*' ignore-parents parent pwd ..

# sudo の後ろでコマンド名を補完する
zstyle ':completion:*:sudo:*' command-path /usr/local/sbin /usr/local/bin \
  /usr/sbin /usr/bin /sbin /bin /usr/X11R6/bin

# ps コマンドのプロセス名補完
zstyle ':completion:*:processes' command 'ps x -o pid,s,args'


##  コマンド履歴
# コマンド履歴を保存する
HISTFILE=~/.zsh_history
HISTSIZE=1000000
SAVEHIST=1000000

#  コマンド履歴をインクリメンタル検索する
bindkey '^r' history-incremental-pattern-search-backward
bindkey '^s' history-incremental-pattern-search-forward

# すでに入力した内容を使ってコマンド履歴を検索する
autoload -Uz history-search-end
zle -N history-beginning-search-backward-end history-search-end
bindkey "^o" history-beginning-search-backward-end

# 同時に起動したzshの間でヒストリを共有する
setopt SHARE_HISTORY

# 同じコマンドをヒストリに残さない
setopt HIST_IGNORE_ALL_DUPS

# ヒストリに保存するときに余分なスペースを削除する
setopt HIST_REDUCE_BLANKS

# スペースから始まるコマンド行はヒストリに残さない
setopt HIST_IGNORE_SPACE


## エイリアス
alias la='ls -a'
alias ll='ls -l'

alias rm='rm -i'
alias cp='cp -i'
alias mv='mv -i'

alias mkdir='mkdir -p'

alias ..='cd ..'
alias ...='cd ../..'
alias ....='cd ../../..'

# グローバルエイリアス
alias -g L='| less'
alias -g H='| head'
alias -g T='| tail'
alias -g G='| grep'

# C で標準出力をクリップボードにコピーする
# mollifier delta blog : http://mollifier.hatenablog.com/entry/20100317/p1
if which pbcopy >/dev/null 2>&1 ; then
  # Mac
  alias -g C='| pbcopy'
elif which xsel >/dev/null 2>&1 ; then
  # Linux
  alias -g C='| xsel --input --clipboard'
elif which putclip >/dev/null 2>&1 ; then
  # Cygwin
  alias -g C='| putclip'
fi

#############################################
# 3章 プラグインの活用
#############################################

## 最近移動したディレクトリに移動する : cdr
## man zshcontrib 「REMEMBERING RECENT DIRECTORIES」
autoload -Uz add-zsh-hook
autoload -Uz chpwd_recent_dirs cdr
add-zsh-hook chpwd chpwd_recent_dirs
# 履歴として保存するディレクトリの最大数を指定する
zstyle ':chpwd:*' recent-dirs-max 200
# cdrコマンドがcdコマンドを兼ねるようにする
zstyle ":chpwd:*" recent-dirs-default true


## 複数のファイルを一括でリネームする : zmv
## man zshcontrib 「OTHER FUNCTIONS」
autoload -Uz zmv
alias zmv='noglob zmv -W'


## バージョン管理システムのリポジトリの情報を表示する : vcs_info
## man zshcontrib 「GATHERING INFORMATION FROM VERSION CONTROL SYSTEMS」
autoload -Uz vcs_info
autoload -Uz add-zsh-hook

zstyle ':vcs_info:*' formats '%F{green}(%s)-[%b]%f'
zstyle ':vcs_info:*' actionformats '%F{red}(%s)-[%b|%a]%f'

function _update_vcs_info_msg() {
  LANG=en_US.UTF-8 vcs_info
  RPROMPT="${vcs_info_msg_0_}"
}
add-zsh-hook precmd _update_vcs_info_msg


## プラグインをまとめて管理する : antigen
## https://github.com/zsh-users/antigen
if [[ -f $HOME/.zsh/antigen/antigen.zsh ]]; then
  source $HOME/.zsh/antigen/antigen.zsh

  ## コマンドラインに色を付ける : zsh-syntax-highlighting
  ## https://github.com/zsh-users/zsh-syntax-highlighting
  antigen bundle zsh-users/zsh-syntax-highlighting


  ## 補完を強化する : zsh-completions
  ## https://github.com/zsh-users/zsh-completions
  antigen bundle zsh-users/zsh-completions src


  ## 4章 外部ツールとの連携
  ## peco/percolのラッパー関数を管理する : anyframe
  ## https://github.com/mollifier/anyframe
  antigen bundle mollifier/anyframe
  # 履歴を実行する
  bindkey '^x^r' anyframe-widget-execute-history
  # 最近移動したディレクトリに移動する(cdrが必要)
  bindkey '^xb' anyframe-widget-cdr
  # Gitのブランチを切り替える
  bindkey '^x^b' anyframe-widget-checkout-git-branch
  # 履歴をコマンドラインに挿入する
  bindkey '^x^p' anyframe-widget-put-history
  # Gitブランチ名をカーソル位置に挿入する
  bindkey '^x^i' anyframe-widget-insert-git-branch
  # プロセスをkillする
  bindkey '^x^k' anyframe-widget-kill

  # tmuxのウィンドウを切り替える
  function peco-tmux() {
    local i=$(tmux lsw | awk '/active.$/ {print NR-1}')
    local f='#{window_index}: #{window_name}#{window_flags} #{pane_current_path}'
    tmux lsw -F "$f" \
      | anyframe-selector-auto "" --initial-index $i \
      | cut -d ':' -f 1 \
      | anyframe-action-execute tmux select-window -t
  }
  zle -N peco-tmux
  bindkey '^xw' peco-tmux


  antigen apply
fi


#############################################
# 4章 外部ツールとの連携
#############################################

## tmuxと連携してディレクトリを移動する : cdd
## https://github.com/m4i/cdd
if [[ -f $HOME/.zsh/cdd/cdd ]]; then
  source $HOME/.zsh/cdd/cdd
  touch $CDD_FILE
  autoload -Uz add-zsh-hook
  add-zsh-hook chpwd _cdd_chpwd
fi


# OS 別の設定
case ${OSTYPE} in
  darwin*)
    #Mac用の設定
    export CLICOLOR=1
    alias ls='ls -G -F'
    ;;
  linux*)
    #Linux用の設定
    alias ls='ls -F --color=auto'
    ;;
esac

# Local Variables:
# mode: Shell-Script
# sh-indentation: 2
# indent-tabs-mode: nil
# sh-basic-offset: 2
# End:
# vim: ft=zsh sw=2 ts=2 et
